@@ -138,7 +138,9 @@ cache_mem <- function(
   # ============================================================================
   DEBUG         <- TRUE
   INITIAL_SIZE  <- 64L
-  COMPACT_LIMIT <- 256L
+  # When compacting, how much space should be reserved? For example, if there
+  # are 75 items in the cache when it is compacted and COMPACT_MULT is 2, then
+  # the data store will be compacted to have a capacity of 150 items.
   COMPACT_MULT  <- 2
   # If TRUE, the data will be kept in the correct atime (for lru) or mtime (for
   # fifo) order each time get() or set() is called, though the metadata log will
@@ -196,20 +198,37 @@ cache_mem <- function(
   # current column-first approach is much, much faster.
   key_idx_map_  <- fastmap()
 
-  key_          <- rep_len(NA_character_, INITIAL_SIZE)
-  value_        <- vector("list",         INITIAL_SIZE)
-  size_         <- rep_len(NA_real_,      INITIAL_SIZE)
-  mtime_        <- rep_len(NA_real_,      INITIAL_SIZE)
-  atime_        <- rep_len(NA_real_,      INITIAL_SIZE)
+  # These values are set in the reset() method.
+  key_          <- NULL
+  value_        <- NULL
+  size_         <- NULL
+  mtime_        <- NULL
+  atime_        <- NULL
 
-  total_n_      <- 0L  # Total number of items
-  total_size_   <- 0   # Total number of bytes used
-  last_idx_     <- 0L  # Most recent (and largest) index used
+  total_n_      <- NULL  # Total number of items
+  total_size_   <- NULL  # Total number of bytes used
+  last_idx_     <- NULL  # Most recent (and largest) index used
 
 
   # ============================================================================
   # Public methods
   # ============================================================================
+
+  reset <- function() {
+    log_(paste0('reset'))
+    key_idx_map_$reset()
+    key_        <<- rep_len(NA_character_, INITIAL_SIZE)
+    value_      <<- vector("list",         INITIAL_SIZE)
+    size_       <<- rep_len(NA_real_,      INITIAL_SIZE)
+    mtime_      <<- rep_len(NA_real_,      INITIAL_SIZE)
+    atime_      <<- rep_len(NA_real_,      INITIAL_SIZE)
+
+    total_n_    <<- 0L
+    total_size_ <<- 0
+    last_idx_   <<- 0L
+    invisible(TRUE)
+  }
+
   get <- function(key, missing = missing_) {
     log_(paste0('get: key "', key, '"'))
     validate_key(key)
@@ -361,12 +380,6 @@ cache_mem <- function(
     invisible(TRUE)
   }
 
-  reset <- function() {
-    log_(paste0('reset'))
-    key_idx_map_$reset()
-    invisible(TRUE)
-  }
-
   prune <- function() {
     log_(paste0('prune'))
 
@@ -508,9 +521,7 @@ cache_mem <- function(
       vapply(keys, remove_one_, TRUE)
     }
 
-    if (last_idx_ > COMPACT_LIMIT  &&  last_idx_ > total_n_ * COMPACT_MULT) {
-      compact_()
-    }
+    compact_()
   }
 
   remove_one_ <- function(key) {
@@ -537,18 +548,17 @@ cache_mem <- function(
   }
 
   compact_ <- function() {
+    if (last_idx_ <= INITIAL_SIZE  ||  last_idx_ <= total_n_ * COMPACT_MULT) {
+      return()
+    }
+
     from_idxs <- key_[seq_len(last_idx_)]
     from_idxs <- !is.na(from_idxs)
     from_idxs <- which(from_idxs)
 
     if (DEBUG) stopifnot(total_n_ == length(from_idxs))
 
-    if (total_n_ == 0L) {
-      message("nothing to compact")
-      return()
-    }
-
-    new_size <- ceiling(total_n_ * COMPACT_MULT)
+    new_size <- max(INITIAL_SIZE, ceiling(total_n_ * COMPACT_MULT))
 
     # Allocate new vectors for metadata.
     new_key_   <- rep_len(NA_character_, new_size)
@@ -614,6 +624,9 @@ cache_mem <- function(
     cat(text, sep = "\n", file = logfile_, append = TRUE)
   }
 
+
+  reset()
+
   # ============================================================================
   # Returned object
   # ============================================================================
